#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@Date:2022/07/19 10:49:15
'''
from decimal import Decimal
import sys
sys.path.append('.')
import json
import datetime
from sqlalchemy import or_, and_, asc, desc
from sqlalchemy.sql import func
from db_logic.db_base import Repository

from models.medicament_models import EntityMedicamentRecord

from Common.Utils import Utils
# from common.utils import Utils

#药剂流转记录业务逻辑类
class BllMedicamentRecord(Repository):
    #_instance_lock = threading.Lock()
    ##实现单例模式
    #def __new__(cls, *args, **kwargs):
    #    if not hasattr(BllMedicamentRecord, "_instance"):
    #        with BllMedicamentRecord._instance_lock:
    #            if not hasattr(BllMedicamentRecord, "_instance"):
    #                BllMedicamentRecord._instance = object.__new__(cls)
    #    return BllMedicamentRecord._instance

    def __init__(self, entityType=EntityMedicamentRecord):
        return super().__init__(entityType)


    # 获取流转记录
    def get_drug_flow_log(self, page_param):
        sql_all = """
        select a.`name`, a.speci, a.bar_code, a.cas_number, a.purity, a.client_id, a.`status`, b.create_date, b.create_user_name from(
            select medicament_id, `name`, speci, bar_code, cas_number, purity, client_id, `status` from rms_medicament
	    ) a RIGHT JOIN(
		    select medicament_id, create_date, create_user_id, create_user_name from rms_medicament_record order by create_date desc
            ) b on b.medicament_id=a.medicament_id
        """
        count_sql = "select count(*) from rms_medicament_record"
        page_param.totalRecords = self.execute(count_sql).fetchone()[0]
        sql_all += ' limit ' + str((page_param.curPage - 1) * page_param.pageRows) + ',' + str(page_param.pageRows)
        data = Utils.msyql_table_model(self.execute(sql_all))
        # print(data)
        return data
        # return self.execute(sql_all)

    # #获取指定用户流转记录
    # def getUserDrugRecordList(self, pageParam, customerId, userId=None):
    #     queryStr = 'select * from RMS_MedicamentRecord r left join RMS_Medicament m on m.MedicamentId=r.MedicamentId and m.CustomerId=:customerId '
    #     queryParams = {"customerId": customerId}
    #     if(userId):
    #         queryStr += ' where CreateUserId=:userId order by CreateDate desc'
    #         queryParams["userId"] = userId
    #     else:
    #         queryStr += ' order by CreateDate desc'
    #     templateList = self.execute(queryStr + ' limit ' + str((pageParam.curPage - 1)
    #                                 * pageParam.pageRows) + ',' + str(pageParam.pageRows), queryParams).fetchall()
    #     pageParam.totalRecords = self.execute(
    #         queryStr.replace('*', 'count(*)'), queryParams).fetchone()[0]
    #     jsonData = Utils.mysqlTable2Model(templateList)
    #     return jsonData

    #获取今日药剂流转信息数量
    def getTodayDrugRecordCount(self, customerId, clientId):
        todayDate = datetime.datetime.now().strftime("%Y-%m-%d")
        # 使用类型进行分组，直接获取入库，领用，归还的数量
        recordListNew = self.session.query(
            self.entityType.record_type,
            func.count('record_id')
            ).select_from(self.entityType).filter(
                EntityMedicamentRecord.customer_id == customerId,
                EntityMedicamentRecord.client_id == clientId,
                func.date_format(EntityMedicamentRecord.create_date, '%Y-%m-%d') == todayDate
            ).group_by(EntityMedicamentRecord.record_type).all()
        return dict(recordListNew)
    

    def get_drug_log_time_name(self, record_type):
        sql_all = f"""
        select a.`name` `name`,a.medicament_id, b.create_date create_date from (

            select `name`, medicament_id from rms_medicament
            ) a RIGHT JOIN(

                select medicament_id, create_date from rms_medicament_record WHERE record_type={record_type} ORDER BY create_date desc LIMIT 1
                ) b on a.medicament_id=b.medicament_id
        """
        return self.execute(sql_all).fetchall()[0]

    # 获取报表统计主页入库记录信息
    def get_drug_record_count(self):
        sql_all = """
            select COUNT(record_id) count_number, record_type from rms_medicament_record GROUP BY record_type
        """
        data = self.execute(sql_all).fetchall()
        data_li = Utils.msyql_table_model(data)
        data_list = []
        for i in data_li:
            name_date = self.get_drug_log_time_name(i["record_type"])
            new_dic = {
                "name": name_date[0],
                "date_time": name_date[2]
            }
            new_dic.update(**i)
            data_list.append(new_dic)
        return data_list

    # 获取报表统计主页试剂用量消耗
    def report_home_drug_useing_classify(self):
        sql_all = """
            select a.count_id count_number, b.`name` from (

                select count(record_id) count_id, medicament_id from rms_medicament_record GROUP BY medicament_id
            ) a LEFT JOIN(
                select `name`, medicament_id from rms_medicament
            ) b on a.medicament_id=b.medicament_id
        """
        data = self.execute(sql_all).fetchall()
        data_number = sum([i[0] for i in data])
        data_li = Utils.msyql_table_model(data)
        data_list = []
        for i in data_li:
            new_dic = {
                "ratio": Utils.classify(i["count_number"], data_number)
            }
            new_dic.update(**i)
            data_list.append(new_dic)
        return data_list
    
    # 获取报表统计主页人员用量消耗
    def report_home_user_use_info(self):
        sql_all = """
            select a.count_id count_number, a.create_user_id, a.create_user_name, b.avatar_url from(

                select count(record_id) count_id, create_user_id,create_user_name  from rms_medicament_record GROUP BY create_user_id
            ) a LEFT JOIN(

            select avatar_url, user_id from rms_user
            ) b on a.create_user_id= b.user_id

        """
        data = self.execute(sql_all).fetchall()
        data_list = Utils.msyql_table_model(data)
        return data_list

    # 公用方法
    def default_data_list(self, sql_all, finds=None):
        """
        参数介绍
        sql_all = 最终执行sql
        finds = 需要聚合的字段
        """
        # 获取数据
        med_data = self.execute(sql_all).fetchall()
        data_list = []
        base_num = 0
        # 数据格式化
        data= Utils.msyql_table_model(med_data)
        # 数据处理，转换float类型，聚合字段，返回新列表
        for d in data:
            new_dic = {}
            for k,v in d.items():
                new_dic[k] = v if not isinstance(v, Decimal) else float(v)
            data_list.append(new_dic)
            if finds:
                print(base_num)
                base_num += float(new_dic.get(finds))
        return data_list, base_num

    # 获取库存消耗数据
    def durg_stock_loss(self, seach_word, page_param):
        sql_all = """
        select c.`name`, c.cas_number,c.speci, c.net_weight_unit, c.purity, c.enter_stock,
            sum(CASE WHEN c.`status`=1 THEN 1 ELSE 0 END) in_stock,
            sum(CASE WHEN c.`status`=2 THEN 1 ELSE 0 END) up_stock,
            sum(CASE WHEN c.`status`=3 THEN 1 ELSE 0 END) no_stock
        from (
            select a.*, b.enter_stock from (
                select medicament_id, `name`, cas_number, speci, purity, net_weight_unit, `status` from rms_medicament
                %s
            ) a LEFT JOIN(
                SELECT medicament_id, sum(CASE WHEN record_type=1 THEN 1 ELSE 0 END) enter_stock from rms_medicament_record GROUP BY medicament_id
            ) b on b.medicament_id = a.medicament_id
        ) c GROUP BY `name`, purity, speci

        """
        # 拼接搜索条件
        if seach_word:
            seach_word = f"%{seach_word}%"
            sql_all = sql_all % f"where `name` like '{seach_word}' or english_name like '{seach_word}'"
        else:
            sql_all = sql_all % ""
        # 查询获取数据条目数
        try:
            count_number = len(self.execute(sql_all).fetchall())
        except Exception:
            count_number = 0
        page_param.totalRecords = count_number
        # 数据分页处理
        sql_all = Utils.sql_paging_assemble(sql_all, page_param)
        # 获取处理后的数据
        data_list, _ = self.default_data_list(sql_all)
        return data_list

    # 试剂用量消耗
    def durg_useing_info(self, seach_word, page_param):
        sql_all = """
        select c.`name`, c.purity, c.speci, c.cas_number, c.net_weight_unit, 
                sum(CASE WHEN `status`=1 THEN 1 ELSE 0 END) on_in, 
                IFNULL(c.c_two,0) use_num,
	            IFNULL(c.sum_use,0) use_level  from (
            select a.*, b.c_two, b.sum_use from(
                select medicament_id, `name`, speci, cas_number, purity, net_weight_unit, `status` from rms_medicament 
                %s
                ) a LEFT JOIN(
                select medicament_id, FORMAT(sum(use_quantity),2) sum_use, sum(CASE WHEN record_type=2 THEN 1 ELSE 0 END) c_two from rms_medicament_record GROUP BY medicament_id
            ) b on a.medicament_id=b.medicament_id 
        ) c GROUP BY `name`, purity, speci
        """
        # 拼接查询条件
        if seach_word:
            seach_word = f"%{seach_word}%"
            sql_all = sql_all % f"where `name` like '{seach_word}' or english_name like '{seach_word}'"
        else:
            sql_all = sql_all % ""
        # 获取数量
        try:
            count_number = len(self.execute(sql_all).fetchall())
        except Exception:
            count_number = 0
        page_param.totalRecords = count_number
        page_sql_all = Utils.sql_paging_assemble(sql_all, page_param)
        # 获取组建后的数据结果
        data, ratio_all = self.default_data_list(page_sql_all, "use_level")
        # 拼接百分比数据
        for i in data:
            i["ratio"] = Utils.classify(i["use_level"], ratio_all)
        return data

    # 人员用量消耗
    def user_use_info(self, page_param, seach_user=None):
        # sql_all = """
        # select c.by_user_name, c.by_user_id, c.sum_use use_number,  IFNULL(c.c_two, 0) use_level from (
        #     select a.*, b.c_two, b.sum_use from(
        #         select medicament_id, by_user_name, by_user_id from rms_medicament %s
        #     ) a LEFT JOIN(
        #         select medicament_id, FORMAT(sum(use_quantity), 2) sum_use, sum(CASE WHEN record_type=2 THEN 1 ELSE 0 END) c_two from rms_medicament_record GROUP BY medicament_id
        #     ) b on a.medicament_id=b.medicament_id 
        # ) c GROUP BY by_user_name, by_user_id
        # """
        sql_all = """
        select a.*, b.role_name from (
            select convert(sum(use_quantity), decimal(10,2)) use_amount,
            sum( CASE WHEN record_type = 2 THEN 1 ELSE 0 END ) use_num,
            create_user_id,
            create_user_name
            from rms_medicament_record %s GROUP BY create_user_id
            ) a LEFT JOIN (
                SELECT role_name, user_id FROM rms_user
            ) b  on a.create_user_id = b.user_id 
        """
        # 查询条件组合
        filter_sql = ""
        if seach_user:
            filter_sql += f"where by_user_name like '%{seach_user}%'"
        if filter_sql:
            sql_all = sql_all % filter_sql
        else:
            sql_all = sql_all % ""
        # 分页总数
        try:
            count_number = len(self.execute(sql_all).fetchall())
        except Exception:
            count_number = 0
        page_param.totalRecords = count_number
        # 数据分页
        sql_all=Utils.sql_paging_assemble(sql_all, page_param)
        data, ratio_all = self.default_data_list(sql_all, "use_amount")
        # 数据处理
        for i in data:
            i["ratio"] = Utils.classify(i["use_amount"], ratio_all)
        return data

    # 使用频率
    def use_frequency(self, page_param, client_id=None, seach_word=None):
        sql_all = """
        select c.`name`, c.purity, c.speci, c.cas_number, c.net_weight_unit, 
			 sum( CASE WHEN c.`status` = 1 THEN 1 ELSE 0 END ) new_on,
			 c.client_id, sum(c.tp_2) sum_tp2, sum(c.tp_3) sum_tp3 from (
                select a.*, IFNULL(b.two_type, 0) tp_2, IFNULL(b.three_type, 0) tp_3 from (
                    select medicament_id, `name`, speci, purity, cas_number, net_weight_unit, `status`, client_id from rms_medicament
                    %s
                ) a LEFT JOIN(
                    select medicament_id, 
                    sum( CASE WHEN record_type = 2 THEN 1 ELSE 0 END ) two_type,
                    sum( CASE WHEN record_type = 3 THEN 1 ELSE 0 END ) three_type
                    from rms_medicament_record GROUP BY medicament_id
                )b on a.medicament_id=b.medicament_id
        ) c GROUP BY `name`, purity, speci
        """
        # 查询条件sql组合
        filter_sql = ""
        if client_id:
            filter_sql += f" client_id='{client_id}' "
        if seach_word:
            if filter_sql:
                filter_sql += "and "
            seach_word = f"%{seach_word}%"
            filter_sql += f" `name` like '{seach_word}' or english_name like '{seach_word}'"
        if filter_sql:
            sql_all = sql_all % f"where {filter_sql}"
        else:
            sql_all = sql_all % ""
        # 查询数据列表长度
        try:
            count_number = len(self.execute(sql_all).fetchall())
        except Exception:
            count_number = 0
        page_param.totalRecords = count_number
        # 处理数据
        sql_all = Utils.sql_paging_assemble(sql_all, page_param)
        data, _ = self.default_data_list(sql_all)
        return data

    def inster_log_list(self):
        import random
        data = list(self.execute("select medicament_id from rms_medicament").fetchall())
        # print(random.choice(data)[0])
        # 
        obj_list = []
        for i in range(100):
            obj = EntityMedicamentRecord(
                client_id='1c39cb24-07f8-11ed-abd4-f47b094925e1',
                client_code='12345',
                medicament_id=random.choice(data)[0],
                record_type=random.randint(1,3),
                is_empty=random.randint(0,1),
                record_remain=random.randint(1,100),
                use_quantity=float(round(random.uniform(0, 50), 2)),
                create_date=Utils.get_str_datetime(),
                create_user_id="c0858e96-d900-11eb-8209-009027e3906b",
                create_user_name="admin"
            )
            obj_list.append(obj)
        self.insert_many(obj_list)

    # #获取药剂最后一次使用余量
    # def getLastRecordRemain(self, drugId):
    #     SQL = """
    #     select RecordRemain from rms_medicamentrecord WHERE MedicamentId='""" + drugId + """' ORDER BY CreateDate DESC LIMIT 1 
    #     """
    #     try:
    #         result = BllMedicamentRecord().execute(SQL)
    #         return float(result.fetchone()[0])
    #     except Exception as e:
    #         print(e)
    #         return 0

    # #删除药剂最后一次领用记录
    # def deleteLastUseRecord(self, drugId):
    #     SQL = """
    #     delete from rms_medicamentrecord WHERE MedicamentId='""" + drugId + """' and RecordType=2  ORDER BY CreateDate DESC LIMIT 1 
    #     """
    #     result = BllMedicamentRecord().executeNoParam(SQL)
    #     return result

# if __name__ == '__main__':
#     import random
#     from db_logic.medicament_record import BllMedicamentRecord
#     # customerId, clientId = None, '8db7e540-070f-11ed-a286-f47b094925e1'
#     # sql = f"update rms_medicament_record set use_quantity={round(random.uniform(1,10), 1)}"
#     from Common.Utils import PageParam
#     page_param = PageParam(1, 10)
#     typ_dic = BllMedicamentRecord().inster_log_list()
#     print(typ_dic)